from __future__ import annotations

import abc
from typing import TYPE_CHECKING

from epigos.utils import image as image_utils

if TYPE_CHECKING:
    from epigos.client import Epigos


class PredictionModel(abc.ABC):
    """
    Prediction Model.

    A generic class to represent model inferences

    :param client: Client to use for interaction with the Epigos server
    :param model_id: Unique internal reference from the Epigos platform for the model
    """

    def __init__(self, client: "Epigos", model_id: str) -> None:
        self._client = client
        self._model_id = model_id

    @abc.abstractmethod
    def _build_url(self) -> str:
        raise NotImplementedError()

    @staticmethod
    def _prepare_image(image_path: str) -> str:
        if image_utils.is_path(image_path):
            image = image_utils.image_to_b64(image_path)
        elif image_utils.is_url(image_path):
            image = image_path
        else:
            raise ValueError(f"Image does not exist at {image_path}!")
        return image



from __future__ import annotations

import typing
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path
from typing import TYPE_CHECKING, Any

import httpx
from tqdm import tqdm
from typing_extensions import deprecated

from epigos import typings
from epigos.data_classes import project as project_data_class
from epigos.dataset import BaseDataset, ClassificationDataset, DetectionDataset
from epigos.typings import BoxFormat
from epigos.utils import image as img_utils
from epigos.utils import logger

from .uploader import Uploader

if TYPE_CHECKING:
    from epigos.client import Epigos

ACCEPTED_IMAGE_FORMATS = ("JPEG", "PNG")
IMAGE_SIZE = (1024, 640)


class Project:
    """
    Project class represents a Project in the Epigos AI.
    """

    def __init__(self, client: "Epigos", project_id: str):
        self._client = client
        self.project_id = project_id

        project = self.get()
        self.name = project.name
        self.project_type = project.project_type
        self.workspace_id = project.workspace_id
        self._uploader = Uploader(self._client, self.project_id, self.project_type)

    @property
    def is_classification(self) -> bool:
        """
        Returns true of project type is classification
        :return: bool
        """
        return self.project_type == typings.ProjectType.classification

    @property
    def is_object_detection(self) -> bool:
        """
        Returns true of project type is object detection
        :return: bool
        """
        return self.project_type == typings.ProjectType.object_detection

    def get(self) -> project_data_class.Project:
        """
        Returns Project object from Epigos AI
        :return:
        """
        url = f"/projects/{self.project_id}/"
        res = self._client.make_get(path=url)
        return project_data_class.Project(
            id=res["id"],
            name=res["name"],
            workspace_id=res["workspaceId"],
            project_type=res["projectType"],
        )

    def upload(
        self,
        image_path: typing.Union[str, Path],
        *,
        annotation_path: typing.Optional[typing.Union[str, Path]] = None,
        box_format: BoxFormat = BoxFormat.pascal_voc,
        batch_name: str = "sdk-upload",
        batch_id: typing.Optional[str] = None,
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        use_folder_as_class_name: bool = False,
        yolo_labels_map: typing.Optional[typing.Dict[int, str]] = None,
    ) -> typing.Dict[str, typing.Any]:
        """
        Upload an image and with or without annotations to the Epigos API.
        :param image_path: Path or directory to images to upload.
        :param annotation_path: Path to annotation file to annotate the image
        :param box_format: Format of annotation to upload.
        Defaults to `pascal_voc`.
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param batch_id: ID of batch to upload to within project.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param use_folder_as_class_name: Use containing folder of image as class name.
        Only used for classification projects.
        :param yolo_labels_map: Class ID to label name mapping for YOLO annotation.
        :return:
        """
        is_file = img_utils.is_path(str(image_path))
        if not is_file:
            raise RuntimeError(f"Provided path does not exist at {image_path}!")

        if batch_id is None:
            batch_id = self._uploader.create_batch(batch_name)

        record = self._uploader.upload(
            batch_id,
            image_path,
            annotation_path=annotation_path,
            use_folder_as_class_name=use_folder_as_class_name,
            box_format=box_format,
            labels_map=labels_map,
            yolo_labels_map=yolo_labels_map,
        )
        return record

    def upload_classification_dataset(
        self,
        images_directory: typing.Union[str, Path],
        batch_name: str = "sdk-upload",
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload dataset containing image classification dataset.
        Image folder names will be used as class names for the images.
        :param images_directory: Path to folder containing images
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        return self._upload_dataset(
            images_directory,
            batch_name=batch_name,
            num_workers=num_workers,
        )

    def upload_coco_dataset(
        self,
        images_directory: typing.Union[str, Path],
        annotations_path: typing.Union[str, Path],
        batch_name: str = "sdk-upload",
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload dataset containing COCO annotations and images.
        :param images_directory: Path to folder containing images
        :param annotations_path: Path to the singel file containing the COCO annotations
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        return self._upload_dataset(
            images_directory,
            annotations_directory=annotations_path,
            box_format=BoxFormat.coco,
            batch_name=batch_name,
            labels_map=labels_map,
            num_workers=num_workers,
        )

    def upload_pascal_voc_dataset(
        self,
        images_directory: typing.Union[str, Path],
        annotations_directory: typing.Union[str, Path],
        batch_name: str = "sdk-upload",
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload dataset containing PASCAL VOC annotations and images.
        :param images_directory: Path to folder containing images.
        :param annotations_directory: Path to directory containing
        Pascal VOC annotations.
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        return self._upload_dataset(
            images_directory,
            annotations_directory=annotations_directory,
            box_format=BoxFormat.pascal_voc,
            batch_name=batch_name,
            labels_map=labels_map,
            num_workers=num_workers,
        )

    def upload_yolo_dataset(
        self,
        images_directory: typing.Union[str, Path],
        annotations_directory: typing.Union[str, Path],
        data_yaml_path: typing.Optional[typing.Union[str, Path]] = None,
        batch_name: str = "sdk-upload",
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload dataset containing YOLO annotations and images.
        :param images_directory: Path to folder containing images
        :param annotations_directory: Path to directory containing YOLO annotations
        :param data_yaml_path: Path to YOLO data configuration file.
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        return self._upload_dataset(
            images_directory,
            annotations_directory=annotations_directory,
            data_yaml_path=data_yaml_path,
            box_format=BoxFormat.yolo,
            batch_name=batch_name,
            labels_map=labels_map,
            num_workers=num_workers,
        )

    @deprecated(
        "Use `upload_coco_dataset`, `upload_yolo_dataset`, `upload_pascal_voc_dataset` "
        "or `upload_classification_dataset` instead."
    )
    def upload_dataset(  # pylint: disable=too-many-arguments
        self,
        data_dir: typing.Union[str, Path],
        *,
        annotations_directory: typing.Optional[typing.Union[str, Path]] = None,
        box_format: BoxFormat = BoxFormat.pascal_voc,
        data_yaml_path: typing.Optional[typing.Union[str, Path]] = None,
        batch_name: str = "sdk-upload",
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload an entire dataset to Epigos API.
        :param data_dir: Path to directory containing images and annotations to upload.
        :param annotations_directory: Path to directory containing
        annotations to upload.
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param box_format: Format of annotation to upload.
        Defaults to `pascal_voc` and only used for object detection projects.
        :param data_yaml_path: Path to YOLO data configuration file.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        return self._upload_dataset(
            data_dir=data_dir,
            annotations_directory=annotations_directory,
            box_format=box_format,
            data_yaml_path=data_yaml_path,
            batch_name=batch_name,
            labels_map=labels_map,
            num_workers=num_workers,
        )

    def _upload_dataset(  # pylint: disable=too-many-arguments
        self,
        data_dir: typing.Union[str, Path],
        *,
        annotations_directory: typing.Optional[typing.Union[str, Path]] = None,
        box_format: BoxFormat = BoxFormat.pascal_voc,
        data_yaml_path: typing.Optional[typing.Union[str, Path]] = None,
        batch_name: str = "sdk-upload",
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        num_workers: int = 4,
    ) -> typing.Iterator[dict[str, Any]]:
        """
        Upload an entire dataset to Epigos API.
        :param data_dir: Path to directory containing images and annotations to upload.
        :param annotations_directory: Path to directory containing
        annotations to upload.
        :param batch_name: name of batch to upload to within project.
        Defaults to `sdk-upload`.
        :param box_format: Format of annotation to upload.
        Defaults to `pascal_voc` and only used for object detection projects.
        :param data_yaml_path: Path to YOLO data configuration file.
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param num_workers: Number of cpu workers to use for uploading.
        :return:
        """
        if not img_utils.is_path(str(data_dir)):
            raise RuntimeError(f"Provided path does not exist at {data_dir}!")

        data_dir = Path(data_dir)

        ds = self._read_dataset_directory(
            data_dir,
            box_format=box_format,
            annotations_directory_path=Path(
                annotations_directory or data_dir / "labels"
            ),
            data_yaml_path=Path(data_yaml_path or data_dir / "data.yaml"),
        )
        num_images = len(ds)

        if not num_images > 0:
            raise RuntimeError(
                "Could not read any images or annotations in the directory provided"
            )

        batch_id = self._uploader.create_batch(batch_name)

        if not labels_map and ds.classes:
            labels_map = self._uploader.create_labels(ds.classes)

        def _upload_file(
            img_path: Path, img_annotations: typing.List[typing.Any]
        ) -> typing.Dict[str, typing.Any]:

            record: typing.Dict[str, typing.Any] = {
                "img_path": img_path,
            }
            try:
                resp = self._uploader.upload(
                    batch_id,
                    img_path,
                    annotations=img_annotations,
                    box_format=box_format,
                    labels_map=labels_map,
                    label_names=ds.classes,
                )
                record["response"] = resp
            except httpx.HTTPError:
                logger.exception(
                    "Error occured while uploading file: %s", img_path, exc_info=True
                )
            return record

        with tqdm(total=num_images, desc="Uploading datasets", colour="green") as pbar:
            with ThreadPoolExecutor(max_workers=num_workers) as executor:
                for result in executor.map(
                    lambda p: _upload_file(*p),
                    ds,
                ):
                    pbar.update()
                    yield result

    def _read_dataset_directory(
        self,
        data_dir: Path,
        *,
        annotations_directory_path: Path,
        data_yaml_path: Path,
        box_format: BoxFormat,
    ) -> BaseDataset:
        """
        Reads dataset directory and returns a mapping for image files
        and its corresponding annotations file.
        """
        if self.is_object_detection:
            return DetectionDataset.from_format(
                box_format=box_format,
                images_directory_path=data_dir,
                annotations_directory_path=annotations_directory_path,
                data_yaml_path=data_yaml_path,
            )

        return ClassificationDataset.from_folder(data_dir)



from .client import Epigos
from .core import ClassificationModel, ObjectDetectionModel, Project
from .exceptions import EpigosException

__all__ = (
    "ClassificationModel",
    "Epigos",
    "EpigosException",
    "ObjectDetectionModel",
    "Project",
)



import typing
from json import JSONDecodeError

import httpx
import tenacity

from .__version__ import __version__
from .core import ClassificationModel, ObjectDetectionModel, Project
from .exceptions import EpigosException
from .utils import logger

BASE_API = "https://api.epigos.ai"
RETRY_STATUS_CODES = [502, 503, 504]


def _retry_on_status_codes(exc: BaseException) -> bool:
    return isinstance(exc, EpigosException) and exc.status_code in RETRY_STATUS_CODES


class Epigos:
    """
    Epigos.

    API client for handling resource request to Epigos API.

    :param api_key: Your epigos.ai workspace api key
    :param base_url: Base url to the epigos api.
    :param timeout: HTTP request timeout in seconds. Defaults to 15 seconds.
    :param retries: Number of times to retry requests. Defaults to 3.
    """

    def __init__(
        self,
        api_key: str,
        base_url: str = BASE_API,
        timeout: float = 15.0,
        retries: int = 3,
    ):
        self._api_key = api_key
        self.client = httpx.Client(
            base_url=httpx.URL(base_url),
            timeout=httpx.Timeout(timeout=timeout),
            headers={
                "Content-Type": "application/json",
                "X-Api-Key": self._api_key,
                "X-Client-Sdk": f"Epigos-SDK/Python; Version: {__version__}",
            },
        )
        self.retry_max_attempts = retries
        self._retry: typing.Optional[tenacity.RetryCallState] = None

    @staticmethod
    def _deserialize(
        response: httpx.Response,
    ) -> typing.Any:
        """
        Deserializes response into an object.

        :param response: Htttp response object to be deserialised
        :returns: deserialized response object
        """
        try:
            json_data = response.json()
        except JSONDecodeError:
            json_data = {"message": response.text}

        if not response.is_success:
            raise EpigosException(
                message=json_data.get("message"),
                details=json_data.get("details") or [],
                status_code=response.status_code,
            )
        return json_data

    def make_request(
        self,
        *,
        path: str,
        method: str,
        json: typing.Optional[typing.Any] = None,
        params: typing.Optional[typing.Dict[str, typing.Any]] = None,
        **kwargs: typing.Any,
    ) -> typing.Any:
        """
        Makes the HTTP request and returns deserialized data

        :param path: Path to method endpoint
        :param method: HTTP Method to call
        :param json: Request body
        :param params: Query parameters in the url
        :returns: Returns the response data from the api
        """
        retryer = tenacity.Retrying(
            stop=tenacity.stop_after_attempt(self.retry_max_attempts),
            wait=tenacity.wait_random_exponential(multiplier=1, max=15),
            reraise=True,
            retry=tenacity.retry_if_exception(_retry_on_status_codes),
            before_sleep=tenacity.before_sleep_log(logger, logger.level),
        )
        for attempt in retryer:
            with attempt:
                attempt.retry_state.fn = self.make_request  # type: ignore[assignment]
                self._retry = attempt.retry_state
                response = self.client.request(
                    method,
                    path,
                    params=httpx.QueryParams(params),
                    json=json,
                    **kwargs,
                )
                return self._deserialize(response)

    def make_post(
        self,
        path: str,
        *,
        json: typing.Optional[typing.Any] = None,
        params: typing.Optional[typing.Dict[str, typing.Any]] = None,
        **kwargs: typing.Any,
    ) -> typing.Any:
        """
        Makes the HTTP POST request and returns deserialized data

        :param path: Path to method endpoint
        :param json: Request body
        :param params: Query parameters in the url
        :returns: Returns the response data from the api
        """
        return self.make_request(
            path=path, method="POST", json=json, params=params, **kwargs
        )

    def make_get(
        self,
        path: str,
        *,
        params: typing.Optional[typing.Dict[str, typing.Any]] = None,
        **kwargs: typing.Any,
    ) -> typing.Any:
        """
        Makes the HTTP GET request and returns deserialized data

        :param path: Path to method endpoint
        :param params: Query parameters in the url
        :returns: Returns the response data from the api
        """
        return self.make_request(path=path, method="GET", params=params, **kwargs)

    def project(self, project_id: str) -> Project:
        """
        Creates an instance of project using the given project_id
        :param project_id: ID of project to load
        :return: Project
        """
        if project_id is None:
            raise ValueError("project_id is required")
        return Project(self, project_id)

    def classification(self, model_id: str) -> ClassificationModel:
        """
        Creates an instance of classification model using the given model ID
        :param model_id: Model to load
        :return: ClassificationModel
        """
        if model_id is None:
            raise ValueError("model_id is required")
        return ClassificationModel(self, model_id)

    def object_detection(self, model_id: str) -> ObjectDetectionModel:
        """
        Creates an instance of object detection model using the given model ID
        :param model_id: Model to load
        :return: ObjectDetectionModel
        """
        if model_id is None:
            raise ValueError("model_id is required")
        return ObjectDetectionModel(self, model_id)



import dataclasses
import typing

from PIL import Image

from epigos.utils.image import b64_to_image


@dataclasses.dataclass
class PredictedClass:
    """
    Predicted Class.

    Represents classification prediction item
    """

    category: str
    confidence: float


@dataclasses.dataclass
class Classification(PredictedClass):
    """
    Classification.

    Represents classification prediction results.
    """

    predictions: list[PredictedClass] = dataclasses.field(default_factory=list)

    def dict(self) -> typing.Dict[str, typing.Any]:
        """
        Return dict representation of the data
        :return: dicts
        """
        return dataclasses.asdict(self)


@dataclasses.dataclass
class DetectedObject:
    """
    Detected Object.

    Represents object detection item
    """

    label: str
    confidence: float
    x: float
    y: float
    width: float
    height: float


@dataclasses.dataclass
class ObjectDetection:
    """
    Object Detection.

    Represents object detection inference results.
    """

    detections: typing.List[DetectedObject] = dataclasses.field(default_factory=list)
    base64_image: typing.Optional[str] = dataclasses.field(default=None, repr=False)

    def __post_init__(self) -> None:
        self._image = b64_to_image(self.base64_image) if self.base64_image else None

    def get_image(self) -> Image.Image:
        """
        Get the decoded image from the API response
        :return: Pil.Image
        """
        if not self._image:
            raise ValueError(
                "No image returned for this prediction. "
                "Set `annotate=True` when making predictions "
                "to return the annotated image"
            )
        return self._image

    def show(self) -> None:
        """
        Displays the Pil.Image
        :return:
        """
        self.get_image().show()

    def dict(self) -> typing.Dict[str, typing.Any]:
        """
        Return dict representation of the data
        :return: dicts
        """
        return dataclasses.asdict(self)



import io
import mimetypes
import typing
from pathlib import Path
from typing import TYPE_CHECKING

import httpx
from PIL import Image, ImageOps

from epigos import typings
from epigos.data_classes.dataset import Classification, Detection
from epigos.dataset import utils as dataset_utils
from epigos.typings import BoxFormat
from epigos.utils import image as img_utils
from epigos.utils import logger

if TYPE_CHECKING:
    from epigos.client import Epigos

ACCEPTED_IMAGE_FORMATS = ("JPEG", "PNG")
DEFAULT_IMAGE_SIZE = (1024, 1024)


class Uploader:
    """
    Uploader is responsible for uploading images and annotations to Epigos AI
    """

    def __init__(
        self, client: "Epigos", project_id: str, project_type: typings.ProjectType
    ) -> None:
        self._client = client
        self._project_id = project_id
        self._project_type = project_type

    def upload(
        self,
        batch_id: str,
        image_path: typing.Union[str, Path],
        annotation_path: typing.Optional[typing.Union[str, Path]] = None,
        annotations: typing.Optional[
            typing.Union[typing.List[Classification], typing.List[Detection]]
        ] = None,
        use_folder_as_class_name: bool = False,
        box_format: BoxFormat = BoxFormat.pascal_voc,
        label_names: typing.Optional[typing.List[str]] = None,
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
        yolo_labels_map: typing.Optional[typing.Dict[int, str]] = None,
    ) -> typing.Dict[str, typing.Any]:
        """
        Upload an image and with or without annotation to the Epigos API.
        :param image_path: Path or directory to images to upload
        :param annotation_path: Path to annotation file to annotate the image
        :param annotations: List of annotations to annotate the image.
        :param batch_id: ID of batch to upload to within project.
        :param use_folder_as_class_name: Use containing folder of image as class name.
        Only used for classification projects.
        :param box_format: Format of annotation to upload.
        Defaults to `pascal_voc`
        :param label_names: List of class names of annotations
        :param labels_map: Class ID of label in Epigos AI to class name mapping.
        :param yolo_labels_map: Class ID to label name mapping for YOLO annotation
        :return:
        """

        image_path = Path(image_path)

        if not img_utils.is_path(str(image_path)):
            raise RuntimeError(f"Provided path does not exist at {image_path}!")

        if self._project_type == typings.ProjectType.classification:
            annotation_path = (
                image_path.parent.name if use_folder_as_class_name else annotation_path
            )

        with Image.open(image_path) as img:
            content_type = mimetypes.guess_type(image_path)[0] or "image/jpeg"
            orig_image_size = img.size

            if img.format not in ACCEPTED_IMAGE_FORMATS:
                raise RuntimeError(f"Image format {img.format} not supported.")

            if img.width > DEFAULT_IMAGE_SIZE[0] or img.height > DEFAULT_IMAGE_SIZE[1]:
                img = ImageOps.contain(img, DEFAULT_IMAGE_SIZE)

            if annotation_path and not annotations:
                annotations = self._read_annotations(
                    image_name=image_path.name,
                    annotation_path=str(annotation_path),
                    orig_image_size=orig_image_size,
                    box_format=box_format,
                    yolo_labels_map=yolo_labels_map,
                )
            record = self._upload_image(
                img, image_path, batch_id=batch_id, content_type=content_type
            )

        if annotations:
            processed_annotations = self._prepare_annotations(
                orig_image_size=orig_image_size,
                resize_img_size=img.size,
                annotations=annotations,
            )
            if not label_names:
                label_names = list({d.class_name for d in annotations})

            record["annotations"] = self._create_annotation(
                record_id=record["id"],
                annotations=processed_annotations,
                label_names=label_names,
                labels_map=labels_map,
            )
        else:
            record["annotations"] = []

        return record

    def create_batch(self, batch_name: str) -> str:
        """
        Create a batch for uploading images to Epigos AI
        :param batch_name:
        :return: batch ID
        """
        batch = self._client.make_post(
            path=f"/projects/{self._project_id}/batches/",
            json={"name": batch_name},
        )
        return str(batch["id"])

    def create_labels(self, names: typing.List[str]) -> typing.Dict[str, str]:
        """
        Creates annotation labels for the given names
        :param names: Label names to create
        :return:
        """
        labels_payload = [{"name": name} for name in names]

        labels = self._client.make_post(
            path=f"/projects/{self._project_id}/annotations/labels/",
            json=labels_payload,
        )
        return {label["name"]: label["id"] for label in labels}

    def _upload_image(
        self, img: Image.Image, image_path: Path, batch_id: str, content_type: str
    ) -> typing.Dict[str, typing.Any]:
        presigned = self._client.make_post(
            path=f"/projects/{self._project_id}/upload/",
            json={"name": image_path.name, "content_type": content_type},
        )

        with io.BytesIO() as fp:
            img.save(fp, format="JPEG")
            content = fp.getvalue()
            img_size = len(content)

            upload_response = httpx.put(
                presigned["uploadUrl"],
                content=content,
                headers={
                    "Content-Type": content_type,
                },
            )
            upload_response.raise_for_status()

        record_payload = {
            "name": image_path.name,
            "batchId": batch_id,
            "height": img.height,
            "width": img.width,
            "contentType": content_type,
            "size": img_size,
            "source": presigned["uri"],
        }
        record = self._client.make_post(
            path=f"/projects/{self._project_id}/datasets/records/",
            json=record_payload,
        )
        return dict(record)

    def _create_annotation(
        self,
        *,
        record_id: str,
        annotations: typing.List[typing.Dict[str, typing.Any]],
        label_names: typing.List[str],
        labels_map: typing.Optional[typing.Dict[str, str]] = None,
    ) -> typing.Optional[typing.List[typing.Any]]:

        payload: dict[str, typing.Any] = {
            "dataset_record_id": record_id,
            "annotations": annotations,
        }

        if not labels_map:
            labels_map = self.create_labels(label_names)

        for annotation in payload["annotations"]:
            annotation["label_id"] = labels_map[annotation["label_id"]]

        annotation_resp = self._client.make_post(
            path=f"/projects/{self._project_id}/annotations/",
            json=payload,
        )
        return list(annotation_resp)

    def _read_annotations(
        self,
        image_name: str,
        annotation_path: str,
        orig_image_size: typing.Tuple[int, int],
        box_format: BoxFormat = BoxFormat.pascal_voc,
        yolo_labels_map: typing.Optional[typing.Dict[int, str]] = None,
    ) -> typing.Union[typing.List[Classification], typing.List[Detection]]:

        if self._project_type == typings.ProjectType.classification:
            # assumes annotation_path is the class name for the image
            return [Classification(class_name=annotation_path)]

        annotations: typing.List[Detection] = []

        if not img_utils.is_path(annotation_path):
            logger.warning("No annotations file found: %s!", annotation_path)
            return annotations

        if box_format == typings.BoxFormat.yolo:
            annotations = dataset_utils.read_yolo_to_coco(
                annotation_path,
                image_size=orig_image_size,
                idx_to_label=yolo_labels_map,
            )
        elif box_format == typings.BoxFormat.coco:
            annotations = dataset_utils.read_single_coco_annotation(
                image_name, annotation_path
            )
        else:
            annotations = dataset_utils.read_pascal_voc_to_coco(
                annotation_path,
            )

        if not annotations:
            logger.warning(
                "No annotations available for %s in file: %s!",
                image_name,
                annotation_path,
            )

        return annotations

    @classmethod
    def _prepare_annotations(
        cls,
        orig_image_size: typing.Tuple[int, int],
        resize_img_size: typing.Tuple[int, int],
        annotations: typing.Union[typing.List[Classification], typing.List[Detection]],
    ) -> typing.List[typing.Dict[str, typing.Any]]:
        scale_boxes = orig_image_size != resize_img_size
        img_wdith, img_height = resize_img_size

        img_scale = (img_wdith, img_height) if scale_boxes else None

        metadata = {"image": {"width": img_wdith, "height": img_height}}
        output = []
        for annot in annotations:
            if isinstance(annot, Classification):
                output.append(
                    {
                        "annotation": {
                            "category": typings.AnnotationCategory.category,
                            "metadata": metadata,
                        },
                        "label_id": annot.class_name,
                    }
                )
            elif isinstance(annot, Detection):
                if img_scale:
                    bbox = dataset_utils.resize_bounding_box(
                        annot.bbox, img_scale, orig_image_size
                    )
                else:
                    bbox = annot.bbox

                output.append(
                    {
                        "annotation": {
                            "category": typings.AnnotationCategory.bounding_box,
                            "left": bbox[0],
                            "top": bbox[1],
                            "width": bbox[2],
                            "height": bbox[3],
                            "metadata": metadata,
                        },
                        "label_id": annot.class_name,
                    }
                )

        return output



import typing


class EpigosException(Exception):
    """
    Epigos Exception

    There was an error returned from making request to Epigos AI

    :param message: response status message received from api
    :param details: response error details.
    :param status_code: HTTP status code received from request.
    """

    def __init__(
        self,
        message: typing.Optional[typing.Any],
        details: typing.Optional[typing.Any],
        status_code: int,
    ) -> None:
        super().__init__(
            f"Error Reason: {message} \n Error Details: {details} "
            f"\n HTTP Status Code: {status_code}"
        )
        self.status_code = status_code
        self.details = details



from epigos.core.base import PredictionModel
from epigos.data_classes.prediction import Classification


class ClassificationModel(PredictionModel):
    """
    Classification Model.

    Manages the model inferences for classification models trained in the platform
    """

    def _build_url(self) -> str:
        return f"/predict/classify/{self._model_id}/"

    def predict(self, image_path: str, confidence: float = 0.7) -> Classification:
        """
        Makes classifcation prediction for the given image.

        :param image_path: Path to image (can be local file or remote url).
        :param confidence: Prediction confidence
        :return: Prediction object
        """
        image = self._prepare_image(image_path)

        data = {"image": image, "confidence": confidence}
        url = self._build_url()
        res = self._client.make_post(path=url, json=data)

        return Classification(
            category=res["category"],
            confidence=res["confidence"],
            predictions=res["predictions"],
        )



import dataclasses
import typing


@dataclasses.dataclass
class Detection:
    """
    Dataclass containing information detection annotations
    """

    bbox: typing.Tuple[int, int, int, int]
    class_name: str
    class_id: typing.Optional[int] = None


@dataclasses.dataclass
class Classification:
    """
    Dataclass containing information image classification annotations
    """

    class_name: str



import base64
import io
import os
import urllib.parse

import httpx
from PIL import Image

ACCEPTED_IMAGE_FORMATS = ["PNG", "JPEG"]


def is_path(file_path: str) -> bool:
    """
    Check whether file path exists
    :param file_path: path of file to check
    :return: Boolean
    """
    return os.path.exists(file_path)


def is_url(url: str) -> bool:
    """
    Check whether a hosted image path is valid
    :param url: URL of image
    :returns: Boolean
    """
    if urllib.parse.urlparse(url).scheme not in ("http", "https"):
        return False

    resp = httpx.head(url, follow_redirects=True)
    return resp.is_success


def check_image_path(image_path: str) -> bool:
    """
    Check whether a local OR remote image path is valid
    :param image_path: local path or URL of image
    :returns: Boolean
    """
    return is_path(image_path) or is_url(image_path)


def image_to_b64(image_path: str) -> str:
    """
    Convert local image file to base64 encoded string
    :param image_path: local path to image
    :return: base64 encoded string
    """
    # open Image in RGB Format
    with Image.open(image_path) as im:
        im = im.convert("RGB")
        buffer = io.BytesIO()
        im.save(buffer, quality=90, format="JPEG")
        # Base64 encode image
        img_bytes = base64.b64encode(buffer.getvalue())

    return img_bytes.decode("ascii")


def b64_to_image(image_str: str) -> Image.Image:
    """
    Convert base64 encoded string to Pil.Image
    :param image_str: base64 encoded string image
    :return: Pil.Image
    """
    image_bytes = base64.b64decode(image_str)
    return Image.open(io.BytesIO(image_bytes))



import logging
import os

EPIGOS_DEBUG = str(os.getenv("EPIGOS_DEBUG") or False).lower() == "true"


def get_logger(name: str = "epigos-ai", verbose: bool = EPIGOS_DEBUG) -> logging.Logger:
    """
    Configure logging for the epigos-ai
    """
    level = logging.INFO if verbose else logging.ERROR
    # create logger
    log = logging.getLogger(name)
    log.setLevel(level)

    # create console handler and set level to debug
    handler = logging.StreamHandler()
    handler.setLevel(level)
    # create formatter
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    # add formatter to handler
    handler.setFormatter(formatter)

    # add handler to logger
    log.addHandler(handler)
    log.propagate = True
    return log


logger = get_logger()



import dataclasses

from epigos import typings


@dataclasses.dataclass
class Project:
    """Project data class"""

    id: str
    workspace_id: str
    name: str
    project_type: typings.ProjectType



from __future__ import annotations

import enum
import typing


class ProjectType(str, enum.Enum):
    """
    Project type

    Enums for types of projects
    """

    object_detection = "object_detection"
    classification = "classification"


class ModelType(str, enum.Enum):
    """
    Model type.

    Enums for types of models supported
    """

    classification = "classification"
    object_detection = "object_detection"


class AnnotationCategory(str, enum.Enum):
    """
    Annotation category

    Enums for types of annotations supported
    """

    category = "category"
    bounding_box = "bounding_box"


class DetectOptions(typing.TypedDict, total=False):
    """
    DetectOptions options used to customize the output.

    :param annotate: If True, it annotates the image with the predicted objects.
    :param stroke_width: Specify bounding boxes border size.
    :param show_prob: If True, detected objects will show detection confidence
        for each object in the image.
    """

    annotate: bool
    stroke_width: typing.Optional[int]
    show_prob: bool


class BoxFormat(str, enum.Enum):
    """
    Bounding Box format

    Enums for types of annotation box format supported
    """

    pascal_voc = "pascal_voc"
    yolo = "yolo"
    coco = "coco"



from typing_extensions import Unpack

from epigos import typings
from epigos.core.base import PredictionModel
from epigos.data_classes.prediction import ObjectDetection


class ObjectDetectionModel(PredictionModel):
    """
    Object Detection Model.

    Manages the model inferences for object detection models trained in the platform
    """

    def _build_url(self) -> str:
        return f"/predict/detect/{self._model_id}/"

    def detect(
        self,
        image_path: str,
        confidence: float = 0.7,
        **kwargs: Unpack[typings.DetectOptions],
    ) -> ObjectDetection:
        """
        Infers detections based on image from specified model and image path.

        :param image_path: Path to image (can be local file or remote url).
        :param confidence: Prediction confidence.
        :param kwargs: Annotation options for the prediction
        :return: ObjectDetection object
        """
        annotate = kwargs.get("annotate") or True
        stroke_width = kwargs.get("stroke_width")
        show_prob = kwargs.get("show_prob") or True

        image = self._prepare_image(image_path)

        data = {
            "image": image,
            "confidence": confidence,
            "annotate": annotate,
            "stroke_width": stroke_width,
            "show_prob": show_prob,
        }
        url = self._build_url()
        res = self._client.make_post(path=url, json=data)

        return ObjectDetection(
            detections=res["detections"], base64_image=res.get("image")
        )



import collections
import json
import os
import typing
import xml.etree.ElementTree as ET
from pathlib import Path

import imagesize
import yaml
from pybboxes import BoundingBox

from epigos.data_classes.dataset import Classification, Detection

IMAGE_FILE_EXTENSIONS = (".jpg", ".jpeg", ".png")


def _extract_bbox_from_element(
    bb: ET.Element,
) -> typing.Optional[typing.Tuple[int, ...]]:
    attrs = ["xmin", "ymin", "xmax", "ymax"]

    bbox_els = (bb.find(attr) for attr in attrs)
    bbox_str = (x.text for x in bbox_els if x is not None)
    bbox = tuple(int(float(v)) for v in bbox_str if v is not None)
    if not len(bbox) == 4 or any((b is None for b in bbox)):
        return None
    return bbox


def _extract_size_from_element(
    size: ET.Element,
) -> typing.Optional[typing.Tuple[int, int]]:
    if not size and not len(size) > 1:
        return None

    attrs = ["width", "height"]
    size_els = (size.find(attr) for attr in attrs)
    size_str = (x.text for x in size_els if x is not None)
    imgs_size = tuple(int(x) for x in size_str if x)

    if not len(imgs_size) > 1:
        return None

    return imgs_size[0], imgs_size[1]


def _read_images_with_extensions(
    directory: typing.Union[str, Path], extensions: typing.List[str]
) -> typing.Dict[str, Path]:
    directory = Path(directory)
    imgs_paths = {p.name: p for ext in extensions for p in directory.glob(f"**/*{ext}")}
    return imgs_paths


def read_pascal_voc_to_coco(
    annotation_file: typing.Union[str, Path]
) -> typing.List[Detection]:
    """
    Reads Pascal VOC [xmin, ymin, xmax, ymax] annotations from file
    and converts it to COCO-style annotations.
    :param annotation_file: Path to annotation file to read.
    :return: List: Contains image metadata and COCO
    bounding boxes (x, y, width, height).
    """
    tree = ET.parse(annotation_file)
    root = tree.getroot()
    size_el = root.find("size")
    image_size = _extract_size_from_element(size_el) if size_el else None

    annotations = []

    for obj in root.iter("object"):
        name_el = obj.find("name")
        bb = obj.find("bndbox")

        label_name = name_el.text if name_el is not None else None
        bbox = _extract_bbox_from_element(bb) if bb is not None else None
        if not bbox or not label_name:
            continue

        coco_bbox = BoundingBox.from_voc(*bbox, image_size=image_size).to_coco(
            return_values=True
        )

        annotations.append(
            Detection(
                bbox=(
                    int(coco_bbox[0]),
                    int(coco_bbox[1]),
                    int(coco_bbox[2]),
                    int(coco_bbox[3]),
                ),
                class_name=label_name,
            )
        )

    return annotations


def _read_yolo_annotation_file(
    annotation_file: str,
) -> typing.List[typing.Tuple[int, float, float, float, float]]:
    """
    Read YOLO annotation file and parse bounding box annotations.

    Args:
        annotation_file (str or Path): Path to the YOLO annotation file.

    Returns:
        List of tuples: Each tuple contains
        (class_index, x_center, y_center, width, height)
        for each object.
    """
    with open(annotation_file, "r", encoding="utf-8") as f:
        lines = f.readlines()

    annotations = []
    for line in lines:
        parts = line.strip().split()
        if len(parts) < 5:
            continue
        class_index = int(parts[0])
        x_center = float(parts[1])
        y_center = float(parts[2])
        width = float(parts[3])
        height = float(parts[4])
        annotations.append((class_index, x_center, y_center, width, height))

    return annotations


def read_yolo_to_coco(
    annotation_file: typing.Union[str, Path],
    image_size: typing.Tuple[int, int],
    idx_to_label: typing.Optional[typing.Dict[int, str]] = None,
) -> typing.List[Detection]:
    """
    Reads YOLO [x1, y1, x2, y2] annotations from file and
    converts it to COCO-style annotations.
    :param annotation_file: Path to annotation file to read.
    :param image_size: Original image size for YOLO annotations
    :param idx_to_label: Class ID to label name mapping for YOLO annotation
    :return: Dict: Contains image metadata and
    COCO bounding boxes (x, y, width, height).
    """
    annotations = []
    yolo_boxes = _read_yolo_annotation_file(str(annotation_file))
    idx_to_label = idx_to_label or {}

    for yolo_box in yolo_boxes:
        label_name = idx_to_label.get(yolo_box[0]) or str(yolo_box[0])
        coco_bbox = BoundingBox.from_yolo(
            *yolo_box[1:], image_size=image_size, strict=False
        ).to_coco(return_values=True)

        annotations.append(
            Detection(
                bbox=(
                    int(coco_bbox[0]),
                    int(coco_bbox[1]),
                    int(coco_bbox[2]),
                    int(coco_bbox[3]),
                ),
                class_name=label_name,
                class_id=yolo_box[0],
            )
        )

    return annotations


def read_yolo_config(file_name: Path) -> typing.Any:
    """Read YOLO configuration file"""
    with open(file_name, "r", encoding="utf-8") as fp:
        cfg = yaml.safe_load(fp)
    return cfg


def read_coco_file(file_name: Path) -> typing.Any:
    """Read COCO annotations file"""
    with open(file_name, "r", encoding="utf-8") as fp:
        dataset = json.load(fp)

    assert isinstance(
        dataset, dict
    ), f"annotation file format {type(dataset)} not supported"
    return dataset


def read_image_folder(
    root_directory_path: Path,
) -> typing.Tuple[
    typing.List[str],
    typing.Dict[str, Path],
    typing.Dict[str, typing.List[Classification]],
]:
    """
    Read directory image classification dataset and return
    dictionary of image paths and the class name.
    :param root_directory_path:
    :return:
    """
    images = _read_images_with_extensions(
        root_directory_path, extensions=list(IMAGE_FILE_EXTENSIONS)
    )
    annotations = {
        p.name: [Classification(class_name=p.parent.name)] for p in images.values()
    }

    classes = sorted(set(os.listdir(root_directory_path)))

    return classes, images, annotations


def _get_image_size(img_path: Path) -> typing.Tuple[int, int]:
    width, height = imagesize.get(str(img_path))
    return width, height


def read_yolo_directory(
    images_directory_path: Path, annotations_directory_path: Path, data_yaml_path: Path
) -> typing.Tuple[
    typing.List[str], typing.Dict[str, Path], typing.Dict[str, typing.List[Detection]]
]:
    """
    Read directory containing YOLO dataset and return dictionary of image paths
    and their corresponding annotation path.
    :param images_directory_path:
    :param annotations_directory_path:
    :param data_yaml_path:
    :return:
    """
    yolo_config = read_yolo_config(data_yaml_path)
    yolo_names = yolo_config.get("names") or {}

    idx_to_label = {}
    if isinstance(yolo_names, list):
        idx_to_label = dict(enumerate(yolo_names))
    elif isinstance(yolo_names, dict):
        idx_to_label = yolo_names

    images = _read_images_with_extensions(
        images_directory_path, extensions=list(IMAGE_FILE_EXTENSIONS)
    )
    yolo_annotations = {
        p.name: read_yolo_to_coco(
            (annotations_directory_path / f"{p.stem}.txt").resolve(),
            _get_image_size(p),
            idx_to_label,
        )
        for p in images.values()
    }
    classes = list(sorted(list(idx_to_label.values())))
    return classes, images, yolo_annotations


def read_pascal_voc_directory(
    images_directory_path: Path,
    annotations_directory_path: Path,
) -> typing.Tuple[
    typing.List[str], typing.Dict[str, Path], typing.Dict[str, typing.List[Detection]]
]:
    """
    Read directory containing Pascal VOC dataset
    and return dictionary of image paths
    and their corresponding annotation path

    :param images_directory_path: Path to directory containing images
    :param annotations_directory_path: Path to directory containing annotations
    :return:
    """
    images = _read_images_with_extensions(
        images_directory_path, extensions=list(IMAGE_FILE_EXTENSIONS)
    )

    pascal_annotations = {
        p.name: read_pascal_voc_to_coco(
            (annotations_directory_path / f"{p.stem}.xml").resolve()
        )
        for p in images.values()
    }

    classes = {
        d.class_name for detections in pascal_annotations.values() for d in detections
    }
    return list(sorted(classes)), images, pascal_annotations


def read_single_coco_annotation(
    image_name: str, annotations_path: typing.Union[str, Path]
) -> typing.List[Detection]:
    """
    Read a COCO file contaning annotations for a single image.
    :param image_name:
    :param annotations_path:
    :type annotations_path:
    :return:
    :rtype:
    """
    dataset = read_coco_file(Path(annotations_path))
    label_to_idx = {
        label["name"]: label["id"] for label in dataset.get("categories") or []
    }
    idx_to_label = {v: k for k, v in label_to_idx.items()}

    filename_to_img_id = {
        img["file_name"]: img["id"] for img in dataset.get("images") or []
    }

    output: typing.List[Detection] = []

    img_id = filename_to_img_id.get(image_name)
    if img_id is None:
        return output

    for ann in dataset.get("annotations") or []:
        if not ann["bbox"]:
            continue
        if ann["image_id"] != img_id:
            continue

        label_name = idx_to_label[ann["category_id"]]
        output.append(
            Detection(
                bbox=(
                    int(ann["bbox"][0]),
                    int(ann["bbox"][1]),
                    int(ann["bbox"][2]),
                    int(ann["bbox"][3]),
                ),
                class_name=label_name,
                class_id=ann["category_id"],
            )
        )
    return output


def read_coco_directory(
    images_directory_path: Path,
    annotations_path: Path,
) -> typing.Tuple[
    typing.List[str], typing.Dict[str, Path], typing.Dict[str, typing.List[Detection]]
]:
    """
    A COCO dataset helper for reading coco annotations and images in a directory.
    :param images_directory_path:
    :param annotations_path:
    :return:
    """
    dataset = read_coco_file(annotations_path)
    label_to_idx = {
        label["name"]: label["id"] for label in dataset.get("categories") or []
    }
    idx_to_label = {v: k for k, v in label_to_idx.items()}

    imgs = {img["id"]: img for img in dataset.get("images") or []}

    imgs_paths = {
        img_id: (images_directory_path / imgs[img_id]["file_name"]).resolve()
        for img_id in imgs
        if Path(imgs[img_id]["file_name"]).suffix.lower() in IMAGE_FILE_EXTENSIONS
    }
    image_id_to_annotations = collections.defaultdict(list)

    for ann in dataset.get("annotations") or []:
        if not ann["bbox"]:
            continue
        label_name = idx_to_label[ann["category_id"]]
        image_id_to_annotations[ann["image_id"]].append(
            Detection(
                bbox=(
                    int(ann["bbox"][0]),
                    int(ann["bbox"][1]),
                    int(ann["bbox"][2]),
                    int(ann["bbox"][3]),
                ),
                class_name=label_name,
                class_id=ann["category_id"],
            )
        )

    images = {img_path.name: img_path for img_path in imgs_paths.values()}
    annotations = {
        img_path.name: image_id_to_annotations[img_id]
        for img_id, img_path in imgs_paths.items()
    }
    classes = list(sorted(list(label_to_idx.keys())))

    return classes, images, annotations


def resize_bounding_box(
    bbox: typing.Tuple[int, int, int, int],
    img_scale: typing.Tuple[int, int],
    orig_image_size: typing.Tuple[int, int],
) -> typing.Tuple[int, int, int, int]:
    """
    Resize bounding box according to img scale and original image size
    :param bbox:
    :param img_scale:
    :param orig_image_size:
    :return:
    """
    scale_x = img_scale[0] / orig_image_size[0]
    scale_y = img_scale[1] / orig_image_size[1]
    bbox = (
        int(bbox[0] * scale_x),
        int(bbox[1] * scale_y),
        int(bbox[2] * scale_x),
        int(bbox[3] * scale_y),
    )
    return bbox



from __future__ import annotations

import abc
import dataclasses
import typing
from pathlib import Path

from epigos import typings
from epigos.data_classes.dataset import Classification, Detection
from epigos.dataset import utils
from epigos.typings import BoxFormat


@dataclasses.dataclass
class BaseDataset(abc.ABC):
    """
    Base dataset class for all datasets
    """

    classes: typing.List[str]
    images: typing.Dict[str, Path]

    def __len__(self) -> int:
        """
        Return the number of images in the dataset.
        :return:
            int: The number of images
        """
        return len(self.images)

    @abc.abstractmethod
    def __iter__(
        self,
    ) -> typing.Iterator[
        typing.Tuple[
            Path, typing.Union[typing.List[Classification], typing.List[Detection]]
        ]
    ]:
        raise NotImplementedError()


@dataclasses.dataclass
class DetectionDataset(BaseDataset):
    """
    Dataclass containing information about object detection dataset
    """

    annotations: typing.Dict[str, typing.List[Detection]]

    def __iter__(self) -> typing.Iterator[typing.Tuple[Path, typing.List[Detection]]]:
        """
        Iterate over the images and annotations in the dataset
        :return:
        """
        for image_name, image_path in self.images.items():
            yield image_path, self.annotations.get(image_name, [])

    @classmethod
    def from_pascal_voc(
        cls,
        images_directory_path: typing.Union[str, Path],
        annotations_directory_path: typing.Union[str, Path],
    ) -> DetectionDataset:
        """
        Read directory containing Pascal VOC dataset
        and return dictionary of image paths
        and their corresponding annotation path

        :param images_directory_path: Path to directory containing images
        :param annotations_directory_path: Path to directory containing annotations
        :return:
        """
        images_directory_path = Path(images_directory_path)
        annotations_directory_path = Path(annotations_directory_path)

        classes, images, pascal_annotations = utils.read_pascal_voc_directory(
            images_directory_path, annotations_directory_path
        )
        return cls(classes=classes, images=images, annotations=pascal_annotations)

    @classmethod
    def from_coco(
        cls,
        images_directory_path: typing.Union[str, Path],
        annotations_path: typing.Union[str, Path],
    ) -> DetectionDataset:
        """
        Read directory containing COCO dataset
        and return dictionary of image paths
        and their corresponding annotation path

        :param images_directory_path: Path to directory containing images
        :param annotations_path: Path to file containing annotations
        :return:
        """
        images_directory_path = Path(images_directory_path)
        annotations_path = Path(annotations_path)

        classes, images, coco_annotations = utils.read_coco_directory(
            images_directory_path, annotations_path
        )
        return cls(classes=classes, images=images, annotations=coco_annotations)

    @classmethod
    def from_yolo(
        cls,
        images_directory_path: typing.Union[str, Path],
        annotations_directory_path: typing.Union[str, Path],
        data_yaml_path: typing.Union[str, Path],
    ) -> DetectionDataset:
        """
        Read directory containing Pascal VOC dataset
        and return dictionary of image paths
        and their corresponding annotation path

        :param images_directory_path: Path to directory containing images
        :param annotations_directory_path: Path to directory containing annotations
        :param data_yaml_path: Path to file containing YOLO data configuration
        :return:
        """
        images_directory_path = Path(images_directory_path)
        annotations_directory_path = Path(annotations_directory_path)
        data_yaml_path = Path(data_yaml_path)

        classes, images, yolo_annotations = utils.read_yolo_directory(
            images_directory_path, annotations_directory_path, data_yaml_path
        )
        return cls(classes=classes, images=images, annotations=yolo_annotations)

    @classmethod
    def from_format(
        cls,
        box_format: typings.BoxFormat,
        images_directory_path: typing.Union[str, Path],
        annotations_directory_path: typing.Union[str, Path],
        data_yaml_path: typing.Union[str, Path],
    ) -> DetectionDataset:
        """
        Reads dataset directory with annotations for a given annotations format
        :param box_format:
        :param images_directory_path:
        :param annotations_directory_path:
        :param data_yaml_path:
        :return:
        """
        if box_format == BoxFormat.coco:
            return cls.from_coco(
                images_directory_path, annotations_path=annotations_directory_path
            )

        if box_format == BoxFormat.pascal_voc:
            return cls.from_pascal_voc(
                images_directory_path, annotations_directory_path
            )

        return cls.from_yolo(
            images_directory_path, annotations_directory_path, data_yaml_path
        )


@dataclasses.dataclass
class ClassificationDataset(BaseDataset):
    """
    Dataclass containing information about image classification dataset
    """

    annotations: typing.Dict[str, typing.List[Classification]]

    def __iter__(
        self,
    ) -> typing.Iterator[typing.Tuple[Path, typing.List[Classification]]]:
        """
        Iterate over the images and annotations in the dataset
        :return:
        """
        for image_name, image_path in self.images.items():
            yield image_path, self.annotations.get(image_name, [])

    @classmethod
    def from_folder(
        cls, images_directory_path: typing.Union[str, Path]
    ) -> ClassificationDataset:
        """
        Read directory image classification dataset and return
        dictionary of image paths and the class name.
        :param images_directory_path:
        :return:
        """
        images_directory_path = Path(images_directory_path)

        classes, images, annotations_ = utils.read_image_folder(images_directory_path)
        return cls(classes=classes, images=images, annotations=annotations_)
